@@ -34,7 +34,7 @@ class MongoDBDataSource(base.DataSource):
         self.expand = expand
 
         self.collection = None
-        self._fields = None
+        self.fields = None
 
     def initialize(self):
         """Initialize Mongo source stream:
@@ -96,13 +96,13 @@ class MongoDBDataSource(base.DataSource):
 
             fields.append(field)
 
-        self._fields = list(fields)
-        return self._fields
+        self.fields = list(fields)
+        return self.fields
 
     def rows(self):
         if not self.collection:
             raise RuntimeError("Stream is not initialized")
-        fields = self.field_names
+        fields = self.fields.names
         iterator = self.collection.find(fields=fields)
         return MongoDBRowIterator(iterator, fields)
 
@@ -110,8 +110,8 @@ class MongoDBDataSource(base.DataSource):
         if not self.collection:
             raise RuntimeError("Stream is not initialized")
         # return MongoDBRowIterator(self.field_names, self.collection.find())
-        if self._fields:
-            fields = self.field_names
+        if self.fields:
+            fields = self.fields.names()
         else:
             fields = None
         iterator = self.collection.find(fields=fields)
@@ -124,11 +124,19 @@ class MongoDBRowIterator(object):
         self.cursor = cursor
         self.field_names = field_names
 
-    def __getitem__(self, index):
-        record = self.cursor.__getitem__(index)
+    def __iter__(self):
+        return self
+
+    def next(self):
+        record = self.cursor.next()
+
+        if not record:
+            raise StopIteration
 
         array = []
 
+        # FIXME: make use of self.expand
+
         for field in self.field_names:
             value = record
             for key in field.split('.'):
@@ -140,6 +148,21 @@ class MongoDBRowIterator(object):
 
         return tuple(array)
 
+def collapse_record(record, parent=None):
+    ret = {}
+    for key, value in record.items():
+        if parent:
+            full_key = parent + "." + key
+        else:
+            full_key = key
+
+        if type(value) == dict:
+            expanded = collapse_record(value, full_key)
+            ret.update(expanded)
+        else:
+            ret[full_key] = value
+    return ret
+
 class MongoDBRecordIterator(object):
     """Wrapper for pymongo.cursor.Cursor to be able to return rows() as tuples and records() as 
     dictionaries"""
@@ -147,27 +170,19 @@ class MongoDBRecordIterator(object):
         self.cursor = cursor
         self.expand = expand
 
-    def __getitem__(self, index):
-        def expand_record(record, parent=None):
-            ret = {}
-            for key, value in record.items():
-                if parent:
-                    full_key = parent + "." + key
-                else:
-                    full_key = key
+    def __iter__(self):
+        return self
 
-                if type(value) == dict:
-                    expanded = expand_record(value, full_key)
-                    ret.update(expanded)
-                else:
-                    ret[full_key] = value
-            return ret
+    def next(self):
+        record = self.cursor.next()
+        
+        if not record:
+            raise StopIteration
 
-        record = self.cursor.__getitem__(index)
         if not self.expand:
             return record
         else:
-            return expand_record(record)
+            return collapse_record(record)
 
 class MongoDBDataTarget(base.DataTarget):
     """docstring for ClassName
@@ -195,7 +210,7 @@ class MongoDBDataTarget(base.DataTarget):
         self.truncate = truncate
 
         self.collection = None
-        self._fields = None
+        self.fields = None
 
     def initialize(self):
         """Initialize Mongo source stream:
@@ -213,6 +228,8 @@ class MongoDBDataTarget(base.DataTarget):
 
         if self.truncate:
             self.collection.remove()
+            
+        self.field_names = self.fields.names()
 
     def append(self, obj):
         if type(obj) == dict:
