@@ -7,11 +7,7 @@ from field_nodes import *
 from source_nodes import *
 from target_nodes import *
 
-__all__ = (
-    "Node",
-    "SourceNode",
-    "TargetNode",
-    "NodeFinished",
+__all__ = [
     
     # Field nodes
     "FieldMapNode",
@@ -43,4 +39,6 @@ __all__ = (
     "RecordListTargetNode",
     "StreamTargetNode",
     "FormattedPrinterNode"
-)
\ No newline at end of file
+]
+
+__all__ += base.__all__
