@@ -5,13 +5,13 @@ import base
 import re
 import brewery
 import brewery.ds as ds
-from brewery.streams import FieldError
+from brewery.common import FieldError
 import itertools
 
 class FieldMapNode(base.Node):
     """Node renames input fields or drops them from the stream.
     """
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "label" : "Field Map",
         "description" : "Rename or drop fields from the stream.",
@@ -24,13 +24,22 @@ class FieldMapNode(base.Node):
             {
                 "name": "drop_fields",
                 "label": "drop fields",
-                "description": "List of fields to be dropped from the stream."
+                "description": "List of fields to be dropped from the stream - incompatible with keep_fields."
+            },
+            {
+                "name": "keep_fields",
+                "label": "keep fields",
+                "description": "List of fields to keep from the stream - incompatible with drop_fields."
             }
         ]
     }
 
-    def __init__(self, map_fields = None, drop_fields = None):
+    def __init__(self, map_fields = None, drop_fields = None, keep_fields=None):
         super(FieldMapNode, self).__init__()
+
+        if drop_fields and keep_fields:
+            raise FieldError('Invalid configuration of FieldMapNode: you cant specify both keep_fields and drop_fields.')
+
         if map_fields:
             self.mapped_fields = map_fields
         else:
@@ -40,6 +49,11 @@ class FieldMapNode(base.Node):
             self.dropped_fields = set(drop_fields)
         else:
             self.dropped_fields = set([])
+
+        if keep_fields:
+            self.kept_fields = set(keep_fields)
+        else:
+            self.kept_fields = set([])
             
         self._output_fields = []
         
@@ -56,7 +70,7 @@ class FieldMapNode(base.Node):
         return self._output_fields
 
     def initialize(self):
-        self.map = brewery.FieldMap(rename = self.mapped_fields, drop = self.dropped_fields)
+        self.map = brewery.FieldMap(rename=self.mapped_fields, drop=self.dropped_fields, keep=self.kept_fields)
         self._output_fields = self.map.map(self.input.fields)
         self.filter = self.map.row_filter(self.input.fields)
 
@@ -70,7 +84,7 @@ class FieldMapNode(base.Node):
 class TextSubstituteNode(base.Node):
     """Substitute text in a field using regular expression."""
     
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "label" : "Text Substitute",
         "description" : "Substitute text in a field using regular expression.",
@@ -153,7 +167,7 @@ class TextSubstituteNode(base.Node):
 class StringStripNode(base.Node):
     """Strip spaces (orother specified characters) from string fields."""
 
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "icon": "string_strip_node",
         "label" : "String Strip",
@@ -218,7 +232,7 @@ class CoalesceValueToTypeNode(base.Node):
 
     """
 
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "icon": "coalesce_value_to_type_node",
         "description" : "Coalesce Value to Type",
@@ -256,9 +270,9 @@ class CoalesceValueToTypeNode(base.Node):
         else:
             fields = self.input.fields
 
-        self.string_fields = list(itertools.ifilter(lambda f: f.storage_type == "string", fields))
-        self.integer_fields = list(itertools.ifilter(lambda f: f.storage_type == "integer", fields))
-        self.float_fields = list(itertools.ifilter(lambda f: f.storage_type == "float", fields))
+        self.string_fields = [f for f in fields if f.storage_type == "string"]
+        self.integer_fields = [f for f in fields if f.storage_type == "integer"]
+        self.float_fields = [f for f in fields if f.storage_type == "float"]
         
         self.string_indexes = self.input.fields.indexes(self.string_fields)
         self.integer_indexes = self.input.fields.indexes(self.integer_fields)
@@ -337,7 +351,7 @@ class ValueThresholdNode(base.Node):
         
     """
     
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "label" : "Value Threshold",
         "description" : "Bin values based on a threshold.",
@@ -475,7 +489,7 @@ class DeriveNode(base.Node):
 
     """
 
-    __node_info__ = {
+    node_info = {
         "label" : "Derive Node",
         "description" : "Derive a new field using an expression.",
         "attributes" : [
@@ -490,21 +504,28 @@ class DeriveNode(base.Node):
                                 "new field value"
             },
             {
-                "name": "field_type",
+                "name": "analytical_type",
                  "description": "Analytical type of the new field",
                  "default": "unknown"
+            },
+            {
+                "name": "storage_type",
+                 "description": "Storage type of the new field",
+                 "default": "unknown"
             }
         ]
     }
 
 
-    def __init__(self, formula = None, field_name = "new_field", field_type = "unknown"):
+    def __init__(self, formula = None, field_name = "new_field", analytical_type = "unknown",
+                        storage_type = "unknown"):
         """Creates and initializes selection node
         """
         super(DeriveNode, self).__init__()
         self.formula = formula
         self.field_name = field_name
-        self.field_type = field_type
+        self.analytical_type = analytical_type
+        self.storage_type = storage_type
         self._output_fields = None
 
     @property
@@ -523,7 +544,8 @@ class DeriveNode(base.Node):
         for field in self.input.fields:
             self._output_fields.append(field)
 
-        new_field = brewery.Field(self.field_name, analytical_type = self.field_type)
+        new_field = brewery.Field(self.field_name, analytical_type = self.analytical_type, 
+                                  storage_type = self.storage_type)
         self._output_fields.append(new_field)
 
     def _eval_expression(self, **record):
@@ -531,8 +553,11 @@ class DeriveNode(base.Node):
 
     def run(self):
         for record in self.input.records():
-            value = self._formula_callable(**record)
-            record[self.field_name] = value
+            if self._formula_callable:
+                record[self.field_name] = self._formula_callable(**record)
+            else:
+                record[self.field_name] = None
+
             self.put_record(record)
 
 class BinningNode(base.Node):
@@ -552,7 +577,7 @@ class BinningNode(base.Node):
         
     """
     
-    __node_info__ = {
+    node_info = {
         "type": "field",
         "label" : "Binning",
         "icon": "histogram_node",
