@@ -16,9 +16,10 @@ class RowListSourceNode(base.SourceNode):
     """Source node that feeds rows (list/tuple of values) from a list (or any other iterable)
     object."""
 
-    __node_info__ = {
+    node_info = {
         "label" : "Row List Source",
         "description" : "Provide list of lists or tuples as data source.",
+        "protected": True,
         "attributes" : [
             {
                  "name": "list",
@@ -51,9 +52,10 @@ class RecordListSourceNode(base.SourceNode):
     """Source node that feeds records (dictionary objects) from a list (or any other iterable)
     object."""
 
-    __node_info__ = {
+    node_info = {
         "label" : "Record List Source",
         "description" : "Provide list of dict objects as data source.",
+        "protected": True,
         "attributes" : [
             {
                  "name": "a_list",
@@ -81,9 +83,8 @@ class RecordListSourceNode(base.SourceNode):
 
     def run(self):
         for record in self.list:
-            self.put_record(record)
-        
-    
+            self.put(record)
+            
 class StreamSourceNode(base.SourceNode):
     """Generic data stream source. Wraps a :mod:`brewery.ds` data source and feeds data to the 
     output.
@@ -94,10 +95,11 @@ class StreamSourceNode(base.SourceNode):
     in visual, web or other stream modelling tools.
     """
     
-    __node_info__ = {
+    node_info = {
         "label" : "Data Stream Source",
         "icon": "row_list_source_node",
         "description" : "Generic data stream data source node.",
+        "protected": True,
         "attributes" : [
             {
                  "name": "stream",
@@ -140,27 +142,30 @@ class CSVSourceNode(base.SourceNode):
     `string` and analytical type is set to `typeless`.
 
     """
-    __node_info__ = {
+    node_info = {
         "label" : "CSV Source",
         "icon": "csv_file_source_node",
         "description" : "Read data from a comma separated values (CSV) file.",
         "attributes" : [
             {
                  "name": "resource",
-                 "description": "File name or URL containing comma separated values"
+                 "description": "File name or URL containing comma separated values",
             },
             {
                  "name": "fields",
                  "description": "fields contained in the file",
+                 "type": "fields"
             },
             {
                  "name": "read_header",
                  "description": "flag determining whether first line contains header or not",
+                 "type": "flag",
                  "default": "True"
             },
             {
                  "name": "skip_rows",
-                 "description": "number of rows to be skipped"
+                 "description": "number of rows to be skipped",
+                 "type": "flag"
             },
             {
                  "name": "encoding",
@@ -176,13 +181,91 @@ class CSVSourceNode(base.SourceNode):
             }
         ]
     }
-    def __init__(self, *args, **kwargs):
+    def __init__(self, resource = None, *args, **kwargs):
         super(CSVSourceNode, self).__init__()
+        self.resource = resource
+        self.args = args
+        self.kwargs = kwargs
+        self.stream = None
+        self.fields = None
+        self._output_fields = None
+        
+    @property
+    def output_fields(self):
+        if not self.stream:
+            raise ValueError("Stream is not initialized")
+
+        if not self._output_fields:
+            raise ValueError("Fields are not initialized")
+
+        return self._output_fields
+
+    def initialize(self):
+        self.stream = ds.CSVDataSource(self.resource, *self.args, **self.kwargs)
+        
+        if self.fields:
+            self.stream.fields = self.fields
+        
+        self.stream.initialize()
+        
+        # FIXME: this is experimental form of usage
+        self._output_fields = self.stream.fields.copy()
+        self._output_fields.retype(self._retype_dictionary)
+
+    def run(self):
+        for row in self.stream.rows():
+            self.put(row)
+            
+    def finalize(self):
+        self.stream.finalize()
+
+class XLSSourceNode(base.SourceNode):
+    """Source node that reads Excel XLS files.
+
+    It is recommended to configure node fields before running. If you do not do so, fields are
+    read from the file header if specified by `read_header` flag. Field storage types are set to
+    `string` and analytical type is set to `typeless`.
+
+    """
+    node_info = {
+        "label" : "XLS Source",
+        "icon": "xls_file_source_node",
+        "description" : "Read data from an Excel (XLS) spreadsheet file.",
+        "attributes" : [
+            {
+                 "name": "resource",
+                 "description": "File name or URL containing comma separated values"
+            },
+            {
+                 "name": "fields",
+                 "description": "fields contained in the file",
+            },
+            {
+                 "name": "sheet",
+                 "description": "Sheet index number (as int) or sheet name (as string)"
+            },
+            {
+                 "name": "read_header",
+                 "description": "flag determining whether first line contains header or not",
+                 "default": "True"
+            },
+            {
+                 "name": "skip_rows",
+                 "description": "number of rows to be skipped"
+            },
+            {
+                 "name": "encoding",
+                 "description": "resource data encoding, by default no conversion is performed"
+            }
+        ]
+    }
+    def __init__(self, *args, **kwargs):
+        super(XLSSourceNode, self).__init__()
         self.args = args
         self.kwargs = kwargs
         self.stream = None
         self._fields = None
-        
+
     @property
     def output_fields(self):
         if not self.stream:
@@ -204,21 +287,22 @@ class CSVSourceNode(base.SourceNode):
     fields = property(__get_fields, __set_fields)
 
     def initialize(self):
-        self.stream = ds.CSVDataSource(*self.args, **self.kwargs)
-        
+        self.stream = ds.XLSDataSource(*self.args, **self.kwargs)
+
         if self._fields:
             self.stream.fields = self._fields
-        
+
         self.stream.initialize()
         self._fields = self.stream.fields
 
     def run(self):
         for row in self.stream.rows():
             self.put(row)
-            
+
     def finalize(self):
         self.stream.finalize()
 
+
 class YamlDirectorySourceNode(base.SourceNode):
     """Source node that reads data from a directory containing YAML files.
     
@@ -232,10 +316,11 @@ class YamlDirectorySourceNode(base.SourceNode):
     
     Optionally one can specify a field where file name will be stored.
     """
-    __node_info__ = {
+    node_info = {
         "label" : "YAML Directory Source",
         "icon": "yaml_directory_source_node",
         "description" : "Read data from a directory containing YAML files",
+        "protected": True,
         "attributes" : [
             {
                  "name": "path",
@@ -298,7 +383,7 @@ class GoogleSpreadsheetSourceNode(base.SourceNode):
     For details on query string syntax see the section on sq under
     http://code.google.com/apis/spreadsheets/reference.html#list_Parameters
     """
-    __node_info__ = {
+    node_info = {
         "label" : "Google Spreadsheet Source",
         "icon": "google_spreadsheet_source_node",
         "description" : "Read data from a Google Spreadsheet.",
@@ -381,3 +466,107 @@ class GoogleSpreadsheetSourceNode(base.SourceNode):
 
     def finalize(self):
         self.stream.finalize()
+
+
+class SQLSourceNode(base.SourceNode):
+    """Source node that reads from a sql table.
+    """
+    node_info = {
+        "label" : "SQL Source",
+        "icon": "sql_source_node",
+        "description" : "Read data from a sql table.",
+        "attributes" : [
+            {
+                 "name": "uri",
+                 "description": "SQLAlchemy URL"
+            },
+            {
+                 "name": "table",
+                 "description": "table name",
+            },
+        ]
+    }
+    def __init__(self, *args, **kwargs):
+        super(SQLSourceNode, self).__init__()
+        self.args = args
+        self.kwargs = kwargs
+        self.stream = None
+        self._fields = None
+        
+    @property
+    def output_fields(self):
+        if not self.stream:
+            raise ValueError("Stream is not initialized")
+
+        if not self.stream.fields:
+            raise ValueError("Fields are not initialized")
+
+        return self.stream.fields
+
+    def __set_fields(self, fields):
+        self._fields = fields
+        if self.stream:
+            self.stream.fields = fields
+
+    def __get_fields(self):
+        return self._fields
+
+    fields = property(__get_fields, __set_fields)
+
+    def initialize(self):
+        self.stream = ds.SQLDataSource(*self.args, **self.kwargs)
+        self.stream.initialize()
+        self._fields = self.stream.fields
+
+    def run(self):
+        for row in self.stream.rows():
+            self.put(row)
+            
+    def finalize(self):
+        self.stream.finalize()
+
+class GeneratorFunctionSourceNode(base.SourceNode):
+    """Source node uses a callable to generate records."""
+
+    node_info = {
+        "label" : "Callable Generator Source",
+        "description" : "Uses a callable as record generator",
+        "protected": True,
+        "attributes" : [
+            {
+                 "name": "function",
+                 "description": "Function (or any callable)"
+            },
+            {
+                 "name": "fields",
+                 "description": "Fields the function generates"
+            },
+            {
+                 "name": "args",
+                 "description": "Function arguments"
+            },
+            {
+                 "name": "kwargs",
+                 "description": "Function key-value arguments"
+            }
+        ]
+    }
+
+    def __init__(self, function=None, fields=None, *args, **kwargs):
+        super(GeneratorFunctionSourceNode, self).__init__()
+
+        self.function = function
+        self.fields = fields
+        self.args = args
+        self.kwargs = kwargs
+
+    @property
+    def output_fields(self):
+        if not self.fields:
+            raise ValueError("Fields are not initialized")
+        return self.fields
+
+    def run(self):
+        for row in self.function(*self.args, **self.kwargs):
+            self.put(row)
+
