@@ -15,7 +15,7 @@ class StreamTargetNode(base.TargetNode):
     in visual, web or other stream modelling tools.
     """
     
-    __node_info__ = {
+    node_info = {
         "label" : "Data Stream Target",
         "icon": "row_list_target_node",
         "description" : "Generic data stream data target node.",
@@ -26,6 +26,7 @@ class StreamTargetNode(base.TargetNode):
             }
         ]
     }
+    
     def __init__(self, stream):
         super(StreamTargetNode, self).__init__()
         self.stream = stream
@@ -54,7 +55,7 @@ class RowListTargetNode(base.TargetNode):
     To get list of fields, ask for `output_fields`.
     """
 
-    __node_info__ = {
+    node_info = {
         "label" : "Row List Target",
         "description" : "Store data as list of tuples",
         "attributes" : [
@@ -88,7 +89,7 @@ class RecordListTargetNode(base.TargetNode):
     
     """
 
-    __node_info__ = {
+    node_info = {
         "label" : "Record List Target",
         "description" : "Store data as list of dictionaries (records)",
         "attributes" : [
@@ -123,7 +124,7 @@ class CSVTargetNode(base.TargetNode):
         * truncate: remove data from file before writing, default: True
         
     """
-    __node_info__ = {
+    node_info = {
         "label" : "CSV Target",
         "description" : "Write rows as comma separated values into a file",
         "attributes" : [
@@ -142,14 +143,15 @@ class CSVTargetNode(base.TargetNode):
         ]
     }
     
-    def __init__(self, *args, **kwargs):
+    def __init__(self, resource = None, *args, **kwargs):
         super(CSVTargetNode, self).__init__()
+        self.resource = resource
         self.kwargs = kwargs
         self.args = args
         self.stream = None
 
     def initialize(self):
-        self.stream = ds.CSVDataTarget(*self.args, **self.kwargs)
+        self.stream = ds.CSVDataTarget(self.resource, *self.args, **self.kwargs)
 
         self.stream.fields = self.input_fields
         self.stream.initialize()
@@ -208,7 +210,7 @@ class FormattedPrinterNode(base.TargetNode):
 
     """
 
-    __node_info__ = {
+    node_info = {
         "label" : "Formatted Printer",
         "icong": "formatted_printer_node",
         "description" : "Print input using a string formatter to an output IO stream",
@@ -237,8 +239,8 @@ class FormattedPrinterNode(base.TargetNode):
             }
         ]
     }
-    def __init__(self, format = None, target = sys.stdout, delimiter = None, header = None,
-                 footer = None):
+    def __init__(self, format=None, target=sys.stdout, delimiter=None, 
+                 header=None, footer=None):
         super(FormattedPrinterNode, self).__init__()
         self.format = format
         
@@ -254,13 +256,15 @@ class FormattedPrinterNode(base.TargetNode):
         self.handle = None
         self.close_handle = False
 
-    def run(self):
+    def initialize(self):
         if type(self.target) == str or type(self.target) == unicode:
             self.handle = open(self.target, "w")
             self.close_handle = True
         else:
             self.handle = self.target
             self.close_handle = False
+
+    def run(self):
         
         names = self.input_fields.names()
 
@@ -273,10 +277,15 @@ class FormattedPrinterNode(base.TargetNode):
                 
             format_string = u"" + u"\t".join(fields)
 
-        if self.header:
+        if self.header is not None:
             self.handle.write(self.header)
             if self.delimiter:
                 self.handle.write(self.delimiter)
+        else:
+            header_string = u"" + u"\t".join(names)
+            self.handle.write(header_string)
+            if self.delimiter:
+                self.handle.write(self.delimiter)
             
         for record in self.input.records():
             self.handle.write(format_string.format(**record).encode("utf-8"))
@@ -297,11 +306,123 @@ class FormattedPrinterNode(base.TargetNode):
             if self.close_handle:
                 self.handle.close()
 
-class DatabaseTableTargetNode(base.TargetNode):
+class PrettyPrinterNode(base.TargetNode):
+    """Target node that will pretty print output as a table.
+    """
+
+    node_info = {
+        "label" : "Pretty Printer",
+        "icong": "formatted_printer_node",
+        "description" : "Print input using a pretty formatter to an output IO stream",
+        "attributes" : [
+            {
+                 "name": "target",
+                 "description": "IO object. If not set then sys.stdout will be used. "
+                                "If it is a string, then it is considered a filename."
+            },
+            {
+                 "name": "max_column_width",
+                 "description": "Maximum column width. Default is unlimited. "\
+                                "If set to None, then it is unlimited."
+            },
+            {
+                 "name": "min_column_width",
+                 "description": "Minimum column width. Default is 0 characters."
+            }# ,
+            #             {
+            #                  "name": "sample",
+            #                  "description": "Number of records to sample to get column width"
+            #             },
+        ]
+    }
+    def __init__(self, target=sys.stdout, max_column_width=None, 
+                 min_column_width=0, sample=None,
+                 print_names=True, print_labels=False):
+
+        super(PrettyPrinterNode, self).__init__()
+
+        self.max_column_width = max_column_width
+        self.min_column_width = min_column_width or 0
+        self.sample = sample
+        self.print_names = print_names
+        self.print_labels = print_labels
+
+        self.target = target
+        self.handle = None
+        self.close_handle = False
+
+    def initialize(self):
+        if type(self.target) == str or type(self.target) == unicode:
+            self.handle = open(self.target, "w")
+            self.close_handle = True
+        else:
+            self.handle = self.target
+            self.close_handle = False
+            
+        self.widths = [0] * len(self.input.fields)
+        self.names = self.input.fields.names()
+
+        if self.print_names:
+            self.labels = [f.label for f in self.input.fields]
+        else:
+            self.labels = [f.label or f.name for f in self.input.fields]
+
+        self._update_widths(self.names)
+        if self.print_labels:
+            self._update_widths(self.labels)
+
+    def _update_widths(self, row):
+        for i, value in enumerate(row):
+            self.widths[i] = max(self.widths[i], len(unicode(value)))
+
+    def run(self):
+
+        rows = []
+
+        for row in self.input.rows():
+            rows.append(row)
+            self._update_widths(row)
+
+        #
+        # Create template
+        #
+        
+        if self.max_column_width:
+            self.widths = [min(w, self.max_column_width) for w in self.widths]
+        self.widths = [max(w, self.min_column_width) for w in self.widths]
+        fields = [u"{%d:%d}" % (i, w) for i,w in enumerate(self.widths)]
+        template = u"|" + u"|".join(fields) + u"|\n"
+
+        field_borders = [u"-"*w for w in self.widths]
+        self.border = u"+" + u"+".join(field_borders) + u"+\n"
+
+        self.handle.write(self.border)
+        if self.print_names:
+            self.handle.write(template.format(*self.names))
+        if self.print_labels:
+            self.handle.write(template.format(*self.labels))
+
+        if self.print_names or self.print_labels:
+            self.handle.write(self.border)
+        
+        for row in rows:
+            self.handle.write(template.format(*row))
+
+        self.handle.write(self.border)
+
+        self.handle.flush()
+
+    def finalize(self):
+        if self.handle:
+            self.handle.flush()
+            if self.close_handle:
+                self.handle.close()
+
+class SQLTableTargetNode(base.TargetNode):
     """Feed data rows into a relational database table.
     """
-    __node_info__ = {
-        "label": "Database Table Target",
+    node_info = {
+        "label": "SQL Table Target",
         "icon": "sql_table_target",
         "description" : "Feed data rows into a relational database table",
         "attributes" : [
@@ -346,15 +467,19 @@ class DatabaseTableTargetNode(base.TargetNode):
     }
 
     def __init__(self, *args, **kwargs):
-        super(DatabaseTableTargetNode, self).__init__()
+        super(SQLTableTargetNode, self).__init__()
         self.kwargs = kwargs
         self.args = args
         self.stream = None
 
+        # FIXME: document this
+        self.concrete_type_map = None
+
     def initialize(self):
         self.stream = ds.SQLDataTarget(*self.args, **self.kwargs)
 
         self.stream.fields = self.input_fields
+        self.stream.concrete_type_map = self.concrete_type_map
         self.stream.initialize()
 
     def run(self):
@@ -362,4 +487,8 @@ class DatabaseTableTargetNode(base.TargetNode):
             self.stream.append(row)
 
     def finalize(self):
+        """Flush remaining records and close the connection if necessary"""
         self.stream.finalize()
+
+# Original name is depreciated
+DatabaseTableTargetNode = SQLTableTargetNode
\ No newline at end of file
