@@ -1,128 +1,27 @@
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 import logging
 import threading
 import traceback
 import sys
-import StringIO
+from brewery.utils import get_logger
+from brewery.nodes import *
+from brewery.common import *
 
 __all__ = [
     "Stream",
-    "StreamRuntimeError",
-    "FieldError",
-    "StreamError",
-    "Pipe"
+    "Pipe",
+    "stream_from_dict",
+    "create_builder"
 ]
 
-class FieldError(Exception):
-    """Exception raised on field incompatibility or missing fields."""
-    pass
+JOIN_TIMEOUT = None
 
-from brewery.nodes import *
-
-class StreamError(Exception):
-    """Exception raised on stream."""
-    pass
-
-class StreamRuntimeError(Exception):
-    """Exception raised when a node fails during `run()` phase.
-
-    Attributes:
-        * `message`: exception message
-        * `node`: node where exception was raised
-        * `exception`: exception that was raised while running the node
-        * `traceback`: stack traceback
-        * `inputs`: array of field lists for each input
-        * `output`: output field list
-    """
-    def __init__(self, message=None, node=None, exception=None):
-        super(StreamRuntimeError, self).__init__()
-        if message:
-            self.message = message
-        else:
-            self.message = ""
-
-        self.node = node
-        self.exception = exception
-        self.traceback = None
-        self.inputs = []
-        self.output = []
-        self.attributes = {}
-
-    def print_exception(self, output=None):
-        """Prints exception and details in human readable form. You can specify IO stream object in
-        `output` parameter. By default text is printed to standard output."""
-
-        if not output:
-            output = sys.stdout
-
-        text = u"stream failed. reason: %s\n" % self.message
-        text += u"exception: %s: \n" % self.exception.__class__.__name__
-
-        text += u"node: %s\n" % self.node
-
-        try:
-            text += unicode(self.exception)
-        except Exception, e:
-            text += u"<unable to get exception string: %s>" % e
-
-        text += "\ntraceback\n"
-
-        try:
-            text += u"".join(self.traceback)
-        except Exception, e:
-            text += u"<unable to get traceback string: %s>" % e
-
-        text += "\n"
-
-        if self.inputs:
-            for i, fields in enumerate(self.inputs):
-                text += "input %i:\n" % i
-                input_text = ""
-                for (index, field) in enumerate(fields):
-                    input_text += u"% 5d %s (storage:%s analytical:%s)\n" \
-                                % (index, field.name, field.storage_type, field.analytical_type)
-                text += unicode(input_text)
-        else:
-            text += "input: none"
-
-        text += "\n"
-
-        if self.output:
-            text += "output:\n"
-            for field in self.output:
-                text += u"    %s (storage:%s analytical:%s)\n" \
-                            % (field.name, field.storage_type, field.analytical_type)
-        else:
-            text += "ouput: none"
-
-        text += "\n"
-
-        if self.attributes:
-            text += "attributes:\n"
-            for name, attribute in self.attributes.items():
-                try:
-                    value = unicode(attribute)
-                except Exception, e:
-                    value = "unable to convert to string (exception: %s)" % e
-                text += "    %s: %s\n" % (name, value)
-        else:
-            text += "attributes: none"
-
-        output.write(text)
-
-    def __str__(self):
-        s = StringIO.StringIO()
-        try:
-            self.print_exception(s)
-            v = s.getvalue()
-        except Exception as e:
-            v = "Unable to print strem exception. Reason: %s (%s)" % (e, type(e))
-        finally:
-            s.close()
-
-        return v
+def stream_from_dict(desc):
+    """Create a stream from dictionary `desc`."""
+    stream = Stream()
+    stream.update(desc)
+    return stream
 
 class SimpleDataPipe(object):
     """Dummy pipe for testing nodes"""
@@ -148,9 +47,8 @@ class SimpleDataPipe(object):
 
     def put_record(self, record):
         """Convenience method that will transform record into a row based on pipe fields."""
-        row = []
-        for field in self.fields.names():
-            row.append(record.get(field))
+        row = [record.get(field) for field in self.fields.names()]
+
         self.put(row)
 
     def put(self, obj):
@@ -334,11 +232,13 @@ class Stream(object):
         self.node_dict = {}
         self.connections = set()
 
+        self.logger = get_logger()
+
         if nodes:
-            if type(nodes) == dict:
+            try:
                 for name, node in nodes.items():
                     self.add(node, name)
-            else:
+            except:
                 raise StreamError("Nodes should be a dictionary, is %s" % type(nodes))
 
         if connections:
@@ -351,19 +251,20 @@ class Stream(object):
         """Returns a node in the stream or node with name. This method is used for coalescing in
         other methods, where you can pass either node name or node object.
         
+        Raises KeyError when node does not exist.
+
         :Parameters:
             * `node` - node object or node name
         """
 
         if type(node) == str or type(node) == unicode:
-            if not node in self.node_dict:
-                raise KeyError("Node with name '%s' does not exist" % node)
             return self.node_dict[node]
         else:
             return node
 
     def add(self, node, name=None):
-        """Add a `node` into the stream."""
+        """Add a `node` into the stream. Does not allow to add named node if node with given name
+        already exists. """
         if name:
             if name in self.node_dict:
                 raise KeyError("Node with name %s already exists" % name)
@@ -421,7 +322,6 @@ class Stream(object):
 
         self.connections.discard((source_node, target_node))
 
-
     def sorted_nodes(self):
         """
         Return topologically sorted nodes.
@@ -492,6 +392,7 @@ class Stream(object):
             raise Exception("Steram has at least one cycle")
 
         return sorted_nodes
+        
     def fork(self, node=None):
         """Creates a construction fork of the stream. Used for constructing streams in functional
         fashion. Example::
@@ -528,33 +429,39 @@ class Stream(object):
         return _StreamFork(self, self.node(node))
 
 
-    def update(self, dictionary):
+    def update(self, nodes = None, connections = None):
         """Adds nodes and connections specified in the dictionary. Dictionary might contain
         node names instead of real classes. You can use this method for creating stream
         from a dictionary that was created from a JSON file, for example.
         """
 
-        nodes = dictionary.get("nodes")
-        connections = dictionary.get("connections")
+        node_dict = node_dictionary()
 
-        class_dict = Node.class_dictionary()
+        # FIXME: use either node type identifier or fully initialized node, not
+        #        node class (Warning: might break some existing code,
+        #        depreciate it first
+        
+        nodes = nodes or {}
+        connections = connections or []
 
         for (name, obj) in nodes.items():
             if isinstance(obj, Node):
                 node_instance = obj
             elif isinstance(obj, type) and issubclass(obj, Node):
+                self.logger.warn("Using classes in Stream.update is depreciated")
                 node_instance = obj()
             else:
                 if not "type" in obj:
-                    raise Exception("Node dictionary has no type key")
+                    raise Exception("Node dictionary has no 'type' key")
                 node_type = obj["type"]
 
-                if node_type in class_dict:
-                    node_class = class_dict[node_type]
+                if node_type in node_dict:
+                    node_class = node_dict[node_type]
                     node_instance = node_class()
+
                     node_instance.configure(obj)
                 else:
-                    raise Exception("No node class of type '%s'" % obj)
+                    raise Exception("No node class of type '%s'" % node_type)
 
             self.add(node_instance, name)
 
@@ -562,22 +469,61 @@ class Stream(object):
             for connection in connections:
                 self.connect(connection[0], connection[1])
 
+    def configure(self, config = {}):
+        """Configure node properties based on configuration. Only named nodes can be configured at the
+        moment.
+
+        `config` is a list of dictionaries with keys: ``node`` - node name, ``parameter`` - node parameter
+        name, ``value`` - parameter value
+        
+        .. warning:
+        
+            This method might change to a list of dictionaries where one 
+            dictionary will represent one node, keys will be attributes.
+        
+        """
+        
+        # FIXME: this is wrong, it should be a single dict per node (or not?)
+        # List of attributes:
+        #     * can reflect a form for configuring whole stream
+        #     * can have attribute order regardless of their node ownership
+        # List of nodes:
+        #     * bundled attributes in single dictioary
+        # FIXME: this is inconsistent with node configuration! node.config()
+        
+        configurations = {}
+
+        # Collect configurations for each node
+        
+        for attribute in config:
+            node_name = attribute["node"]
+            attribute_name = attribute["attribute"]
+            value = attribute.get("value")
+
+            if not node_name in configurations:
+                config = {}
+                configurations[node_name] = config
+            else:
+                config = configurations[node_name]
+
+            config[attribute_name] = value
+
+        # Configure nodes
+
+        for (node_name, config) in configurations.items():
+            node = self.node(node_name)
+            node.configure(config)
+        
     def node_targets(self, node):
         """Return nodes that `node` passes data into."""
-        nodes = []
         node = self.node(node)
-        for connection in self.connections:
-            if connection[0] == node:
-                nodes.append(connection[1])
+        nodes =[conn[1] for conn in self.connections if conn[0] == node]
         return nodes
 
     def node_sources(self, node):
         """Return nodes that provide data for `node`."""
-        nodes = []
         node = self.node(node)
-        for connection in self.connections:
-            if connection[1] == node:
-                nodes.append(connection[0])
+        nodes =[conn[0] for conn in self.connections if conn[1] == node]
         return nodes
 
     def _initialize(self):
@@ -590,23 +536,23 @@ class Stream(object):
         
         """
 
-        logging.info("initializing stream")
-        logging.debug("sorting nodes")
+        self.logger.info("initializing stream")
+        self.logger.debug("sorting nodes")
         sorted_nodes = self.sorted_nodes()
         self.pipes = []
 
-        logging.debug("flushing pipes")
+        self.logger.debug("flushing pipes")
         for node in sorted_nodes:
             node.inputs = []
             node.outputs = []
 
         # Create pipes and connect nodes
         for node in sorted_nodes:
-            logging.debug("creating pipes for node %s" % node)
+            self.logger.debug("creating pipes for node %s" % node)
 
             targets = self.node_targets(node)
             for target in targets:
-                logging.debug("  connecting with %s" % (target))
+                self.logger.debug("  connecting with %s" % (target))
                 pipe = Pipe()
                 node.add_output(pipe)
                 target.add_input(pipe)
@@ -614,18 +560,18 @@ class Stream(object):
 
         # Initialize fields
         for node in sorted_nodes:
-            logging.debug("initializing node of type %s" % node.__class__)
-            logging.debug("  node has %d inputs and %d outputs"
+            self.logger.debug("initializing node of type %s" % node.__class__)
+            self.logger.debug("  node has %d inputs and %d outputs"
                                 % (len(node.inputs), len(node.outputs)))
             node.initialize()
 
             # Ignore target nodes
             if isinstance(node, TargetNode):
-                logging.debug("  node is target, ignoring creation of output pipes")
+                self.logger.debug("  node is target, ignoring creation of output pipes")
                 continue
 
             fields = node.output_fields
-            logging.debug("  node output fields: %s" % fields.names())
+            self.logger.debug("  node output fields: %s" % fields.names())
             for output_pipe in node.outputs:
                 output_pipe.fields = fields
 
@@ -649,42 +595,42 @@ class Stream(object):
     def _run(self):
 
 
-        logging.info("running stream")
+        self.logger.info("running stream")
 
         threads = []
         sorted_nodes = self.sorted_nodes()
 
-        logging.debug("launching threads")
+        self.logger.debug("launching threads")
         for node in sorted_nodes:
-            logging.debug("launching thread for node %s" % node)
+            self.logger.debug("launching thread for node %s" % node_label(node))
             thread = _StreamNodeThread(node)
             thread.start()
             threads.append((thread, node))
 
         self.exceptions = []
         for (thread, node) in threads:
-            logging.debug("joining thread for %s" % node)
+            self.logger.debug("joining thread for %s" % node_label(node))
             while True:
-                thread.join(0.2)
+                thread.join(JOIN_TIMEOUT)
                 if thread.isAlive():
                     pass
-                    # logging.debug("thread join timed out")
+                    # self.logger.debug("thread join timed out")
                 else:
                     if thread.exception:
                         self._add_thread_exception(thread)
                     else:
-                        logging.debug("thread joined")
+                        self.logger.debug("thread joined")
                     break
                 if self.exceptions:
-                    logging.info("node exception occured, trying to kill threads")
+                    self.logger.info("node exception occured, trying to kill threads")
                     self.kill_threads()
 
         if self.exceptions:
-            logging.info("run finished with exception")
+            self.logger.info("run finished with exception")
             # Raising only first exception found
             raise self.exceptions[0]
         else:
-            logging.info("run finished sucessfully")
+            self.logger.info("run finished sucessfully")
 
     def _add_thread_exception(self, thread):
         """Create a StreamRuntimeError exception object and fill attributes with all necessary
@@ -694,11 +640,7 @@ class Stream(object):
         exception = StreamRuntimeError(node=node, exception=thread.exception)
 
         exception.traceback = thread.traceback
-
-        array = []
-        for pipe in node.inputs:
-            array.append(pipe.fields)
-        exception.inputs = array
+        exception.inputs = [pipe.fields for pipe in node.inputs]
 
         if not isinstance(node, TargetNode):
             try:
@@ -727,16 +669,19 @@ class Stream(object):
 
 
     def kill_threads(self):
-        logging.info("killing threads")
+        self.logger.info("killing threads")
 
     def _finalize(self):
-        logging.info("finalizing nodes")
+        self.logger.info("finalizing nodes")
 
         # FIXME: encapsulate finalization in exception handler, collect exceptions
         for node in self.sorted_nodes():
-            logging.debug("finalizing node %s" % node)
+            self.logger.debug("finalizing node %s" % node_label(node))
             node.finalize()
-
+def node_label(node):
+    """Debug label for a node: node identifier with python object id."""
+    return "%s(%s)" % (node.identifier() or str(type(node)), id(node))
+    
 class _StreamNodeThread(threading.Thread):
     def __init__(self, node):
         """Creates a stream node thread.
@@ -751,34 +696,36 @@ class _StreamNodeThread(threading.Thread):
         self.node = node
         self.exception = None
         self.traceback = None
+        self.logger = get_logger()
 
     def run(self):
         """Wrapper method for running a node"""
-        logging.debug("%s: start" % self)
+        
+        label = node_label(self.node)
+        self.logger.debug("%s: start" % label)
         try:
             self.node.run()
-        except NodeFinished, e:
-            logging.info("node %s finished" % (self.node))
-        except Exception, e:
-            logging.info("node %s failed: %s" % (self.node, e.__class__.__name__))
+        except NodeFinished as e:
+            self.logger.info("node %s finished" % (label))
+        except Exception as e:
             tb = sys.exc_info()[2]
-            self.traceback = traceback.format_list(traceback.extract_tb(tb))
-            print self.traceback
+            self.traceback = tb
+
+            self.logger.debug("node %s failed: %s" % (label, e.__class__.__name__), exc_info=sys.exc_info)
             self.exception = e
-            del tb
 
         # Flush pipes after node is finished
-        logging.debug("%s: finished" % self)
-        logging.debug("%s: flushing outputs" % self)
+        self.logger.debug("%s: finished" % label)
+        self.logger.debug("%s: flushing outputs" % label)
         for pipe in self.node.outputs:
             if not pipe.closed():
                 pipe.done_sending()
-        logging.debug("%s: flushed" % self)
-        logging.debug("%s: stopping inputs" % self)
+        self.logger.debug("%s: flushed" % label)
+        self.logger.debug("%s: stopping inputs" % label)
         for pipe in self.node.inputs:
             if not pipe.closed():
                 pipe.done_sending()
-        logging.debug("%s: stopped" % self)
+        self.logger.debug("%s: stopped" % self)
 
 class _StreamFork(object):
     """docstring for StreamFork"""
@@ -817,7 +764,7 @@ class _StreamFork(object):
         such as `joins`.
         
         """
-
+        raise NotImplementedError
         # if type(obj) == StreamFork:
         #     node = obj.node
         # else:
@@ -828,18 +775,16 @@ class _StreamFork(object):
         # merge = MergeNode(**kwargs)
         # self.stream.append(merge)
         # self.stream.connect()
-        pass
 
     def append(self, obj):
         """Appends data from nodes using AppendNode"""
-        pass
+        raise NotImplementedError 
 
     def __getattr__(self, name):
         """Returns node class"""
-        class_dict = Node.class_dictionary()
-
-        if not name in class_dict:
-            raise AttributeError(name)
+        # FIXME: use create_node here
+        
+        class_dict = node_dictionary()
 
         node_class = class_dict[name]
 
@@ -856,3 +801,10 @@ class _StreamForkConstructor(object):
         node = self.node_class(*args, **kwargs)
         self.fork += node
         return self.fork
+
+def create_builder():
+    """Creates a stream builder for incremental stream building."""
+    stream = Stream()
+    return stream.fork()
+    
+    
\ No newline at end of file
