@@ -24,14 +24,14 @@ class NodesTestCase(unittest.TestCase):
             pipe.put([i, float(i)/4, "item-%s" % i, custom])
 
     def test_node_subclasses(self):
-        nodes = brewery.streams.Node.subclasses()
+        nodes = brewery.nodes.node_dictionary().values()
         self.assertIn(brewery.nodes.CSVSourceNode, nodes)
         self.assertIn(brewery.nodes.AggregateNode, nodes)
         self.assertIn(brewery.nodes.ValueThresholdNode, nodes)
         self.assertNotIn(brewery.streams.Stream, nodes)
 
     def test_node_dictionary(self):
-        d = brewery.nodes.Node.class_dictionary()
+        d = brewery.nodes.node_dictionary()
         self.assertIn("aggregate", d)
         self.assertIn("csv_source", d)
         self.assertIn("csv_target", d)
@@ -104,7 +104,7 @@ class NodesTestCase(unittest.TestCase):
         node.drop_field("q")
         self.initialize_node(node)
 
-        self.assertEqual(['index', 'str', 'custom'], node.output_field_names)
+        self.assertEqual(['index', 'str', 'custom'], node.output_fields.names())
         
         node.run()
         
@@ -202,10 +202,7 @@ class NodesTestCase(unittest.TestCase):
         self.assertAllRows()
 
     def record_results(self):
-        results = []
-        for record in self.output.records():
-            results.append(record)
-        return results
+        return [r for r in self.output.records()]
 
     def initialize_node(self, node):
         node.initialize()
@@ -218,10 +215,10 @@ class NodesTestCase(unittest.TestCase):
         self.create_distinct_sample()
 
         node.key_fields = ["type"]
-        node.add_aggregation("id", ["sum"])
+        node.add_measure("id", ["sum"])
         self.initialize_node(node)
         
-        fields = node.output_field_names
+        fields = node.output_fields.names()
         a = ['type', 'id_sum', 'id_min', 'id_max', 'id_average', 'record_count']
         
         self.assertEqual(a, fields)
@@ -248,10 +245,10 @@ class NodesTestCase(unittest.TestCase):
         self.create_distinct_sample()
 
         # Setup node
-        node.add_aggregation("id", ["sum"])
+        node.add_measure("id", ["sum"])
         self.initialize_node(node)
 
-        fields = node.output_field_names
+        fields = node.output_fields.names()
         a = ['id_sum', 'id_min', 'id_max', 'id_average', 'record_count']
         self.assertEqual(a, fields)
 
@@ -374,6 +371,7 @@ class NodesTestCase(unittest.TestCase):
         self.initialize_node(node)
         node.run()
         node.finalize()
+
         val = sum([row[4] for row in self.output.buffer])
         self.assertEqual(49500, val)
 
@@ -533,4 +531,27 @@ class NodesTestCase(unittest.TestCase):
         self.assertEqual(5, len(self.output.buffer[0]))
         self.assertEqual(input_len, len(self.output.buffer)) 
         
-        
\ No newline at end of file
+    def test_generator_function(self):
+        node = brewery.nodes.GeneratorFunctionSourceNode()
+        def generator(start=0, end=10):
+            for i in range(start,end):
+                yield [i]
+                
+        node.function = generator
+        node.fields = brewery.metadata.FieldList(["i"])
+        node.outputs = [self.output]
+
+        self.initialize_node(node)
+        node.run()
+        node.finalize()
+        self.assertEqual(10, len(self.output.buffer))
+
+        self.output.buffer = []
+        node.args = [0,5]
+        self.initialize_node(node)
+        node.run()
+        node.finalize()
+        self.assertEqual(5, len(self.output.buffer))
+        a = [row[0] for row in self.output.buffer]
+        self.assertEqual([0,1,2,3,4], a)
+
